--TODO:
-- Heartbeat tracking. Show duration passed as green until DR is possible, show probability of being broken.

local DR_Players = {}
rCCPortraits_Frame = CreateFrame("Frame", nil, UIParent);
rCCPortraits_LabelFrame = CreateFrame("Frame", nil, UIParent);
local lastTrack = 99999999999;

function rCCPortraits_RegisterEvents()
    rCCPortraits_Frame:SetScript("OnUpdate", rCCPortraits_SetVars);
    rCCPortraits_Frame:SetScript("OnEvent", rCCPortraits_OnEvent)
    rCCPortraits_Frame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
end

function rCCPortraits_SetVars()
  if(GetSpellBookItemName(1, "spell") ~= nil) then
    if(rCCPortraits == nil) then
      --no vars saved.
      rCCPortraits = {};
      rCCPortraits.rCCPortraits_Anchor = "Blizzard";
    end
    rCCPortraits_Frame:SetScript("OnUpdate", rCCPortraits_OnUpdate);
    rCCPortraits_CreateOptions();
    rCCPortraits_CreateTargetFrame();
  end
end

function rCCPortraits_OnEvent()
  local _, eventType, _,
      _, _, _, _,
      destGUID, _, _, _,
      spellId, spellName, _, auraType, _ = CombatLogGetCurrentEventInfo();
      --TODO: Replace  spellName duration DB with spellID (Downranking)
      if(auraType == "DEBUFF") then
        --TODO: UnitType (No DR on non player units)
        if(eventType == "SPELL_AURA_APPLIED") then
          rCCPortraits_SetDR(destGUID, spellName);
        elseif(eventType == "SPELL_AURA_REFRESH") then
          rCCPortraits_RefreshCC(destGUID, spellName);
        elseif(eventType == "SPELL_AURA_REMOVED") then
          rCCPortraits_SetDRFade(destGUID, spellName);
        end
      end
      if(eventType == "UNIT_DIED") then
        if(DR_Players[destGUID] ~= nil) then
          DR_Players[destGUID] = nil;
        end
      end
end

function rCCPortraits_OnUpdate()
  if(DR_Players ~= nil) then
    if(lastTrack) < GetTime() then
      DR_Players = {}
    end
    if(UnitExists("target") and DR_Players[UnitGUID("target")] ~= nil) then
      local texture, time, duration = rCCPortraits_GetLongestCC(UnitGUID("target"));
      rCCPortraits_ToggleTexture(time - 20 + duration > GetTime(), texture, time, duration);
    else
      rCCPortraits_ToggleTexture(false);
    end
  end
end

function rCCPortraits_SetDR(player, spell)
    local hasDR = rCCPortraits_DRCategories[spell];
    if(hasDR ~= nil) then
      rCCPortraits_Tracker_Insert(player, hasDR, spell)
      lastTrack = GetTime() + 35;
      return true;
    end
    return false;
end

function rCCPortraits_RefreshCC(player, spell)
    local hasDR = rCCPortraits_DRCategories[spell];
    if(hasDR ~= nil) then
      rCCPortraits_Tracker_Refresh(player, hasDR, spell)
      lastTrack = GetTime() + 35;
      return true;
    end
    return false;
end

function rCCPortraits_SetDRFade(player, spell)
    local hasDR = rCCPortraits_DRCategories[spell];
    if(hasDR ~= nil) then
      if(DR_Players[player] == nil) then
        rCCPortraits_Tracker_Insert(player, hasDR, spell);
      end
      rCCPortraits_Tracker_Remove(player, hasDR, spell)
      lastTrack = GetTime() + 15;
      return true;
    end
    return false;
end

function rCCPortraits_Tracker_Insert(player, type, spell)
  if(rCCPortraits_Tracker_Test(player, type, spell, 0) == nil) then
    DR_Players[player].dr[type].time = GetTime() + 20;
    DR_Players[player].dr[type].duration = rCCPortraits_GetDuration(player, type, spell);
    DR_Players[player].dr[type].texture = rCCPortraits_SpellTextures[spell];
  end
end

function rCCPortraits_Tracker_Refresh(player, type, spell)
  if(rCCPortraits_Tracker_Test(player, type, spell, 1) == nil) then
    if(rCCPortraits_GetDRPenalty(DR_Players[player].dr[type].count) == 0) then
      return
    end
    DR_Players[player].dr[type].time = GetTime() + 20;
    DR_Players[player].dr[type].duration = rCCPortraits_GetDuration(player, type, spell);
    DR_Players[player].dr[type].texture = rCCPortraits_SpellTextures[spell];
  end
end

function rCCPortraits_Tracker_Remove(player, type, spell)
  DR_Players[player].dr[type].time = GetTime();
  DR_Players[player].dr[type].duration = 0;
end

function rCCPortraits_Tracker_Test(player, type, spell, count)
  if(DR_Players[player] == nil) then
    DR_Players[player] = {}
    DR_Players[player].dr = {}
    for i = 1, rCCPortraits_DRCategoryCount do
      DR_Players[player].dr[i] = {
        time = 0;
        duration = 0;
        texture = "";
        count = 0;
      };
    end
  end
  if(DR_Players[player].dr[type].time + 15 > GetTime()) then
    if(DR_Players[player].dr[type].count < 3) then
      DR_Players[player].dr[type].count = DR_Players[player].dr[type].count + 1;
    else
      return false
    end
  else
    DR_Players[player].dr[type].count = count;
  end
end

function rCCPortraits_GetLongestCC(player)
  local texture;
  local time = 0;
  local duration = 0;
  for i=1,rCCPortraits_DRCategoryCount do
    if(DR_Players[player].dr[i].time > time) then
      time = DR_Players[player].dr[i].time;
      texture = "Interface\\Icons\\" .. DR_Players[player].dr[i].texture;
      duration = DR_Players[player].dr[i].duration;
    end
  end
  return texture, time, duration;
end

function rCCPortraits_GetDRPenalty(count)
  if(count == 0) then
    return 1;
  elseif(count == 1) then
    return 0.5;
  elseif(count == 2) then
    return 0.25;
  else
    return 0;
  end
end

function rCCPortraits_GetDuration(player, type, spell)
  local dur = math.floor(rCCPortraits_SpellDurations[spell] * rCCPortraits_GetDRPenalty(DR_Players[player].dr[type].count) + 0.5);
  if(dur > 20) then
    return 20;
  else
    return dur;
  end
end

rCCPortraits_RegisterEvents();
